@@ -80,13 +80,12 @@ TALER_BANK_make_auth_header_ (const struct TALER_BANK_AuthenticationData *auth)
     authh = append (authh,
                     "X-Taler-Bank-Password",
                     auth->details.basic.password);
-    break;
+    return authh;    
   }
-  return authh;
+  return NULL;
 }
 
 
-
 /**
  * Obtain the URL to use for an API request.
  *
