@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2016 GNUnet e.V.
+  Copyright (C) 2016, 2017 GNUnet e.V.
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -39,27 +39,69 @@ run (void *cls)
   int *resultp = cls;
   static struct TBI_Command commands[] =
   {
+    { .oc = TBI_OC_HISTORY,
+      .label = "history-0",
+      .details.history.account_number = 1,
+      .details.history.direction = TALER_BANK_DIRECTION_BOTH,
+      .details.history.start_row_ref = NULL,
+      .details.history.num_results = 1 },
     /* Add EUR:5.01 to account 1 */
     { .oc = TBI_OC_ADMIN_ADD_INCOMING,
-      .label = "deposit-1",
+      .label = "debit-1",
       .details.admin_add_incoming.expected_response_code = MHD_HTTP_OK,
       .details.admin_add_incoming.credit_account_no = 1,
       .details.admin_add_incoming.debit_account_no = 2,
-      .details.admin_add_incoming.amount = "PUDOS:5.01" },
+      .details.admin_add_incoming.exchange_base_url = "https://exchange.net/",
+      .details.admin_add_incoming.amount = "KUDOS:5.01" },
     /* Add EUR:3.21 to account 3 */
+    { .oc = TBI_OC_HISTORY,
+      .label = "history-1c",
+      .details.history.account_number = 1,
+      .details.history.direction = TALER_BANK_DIRECTION_CREDIT,
+      .details.history.start_row_ref = NULL,
+      .details.history.num_results = 5 },
+    { .oc = TBI_OC_HISTORY,
+      .label = "history-1d",
+      .details.history.account_number = 1,
+      .details.history.direction = TALER_BANK_DIRECTION_DEBIT,
+      .details.history.start_row_ref = NULL,
+      .details.history.num_results = 5 },
     { .oc = TBI_OC_ADMIN_ADD_INCOMING,
-      .label = "deposit-2",
+      .label = "debit-2",
       .details.admin_add_incoming.expected_response_code = MHD_HTTP_OK,
       .details.admin_add_incoming.credit_account_no = 3,
       .details.admin_add_incoming.debit_account_no = 2,
-      .details.admin_add_incoming.amount = "PUDOS:3.21" },
+      .details.admin_add_incoming.exchange_base_url = "https://exchange.org/",
+      .details.admin_add_incoming.amount = "KUDOS:3.21" },
+    { .oc = TBI_OC_ADMIN_ADD_INCOMING,
+      .label = "credit-2",
+      .details.admin_add_incoming.expected_response_code = MHD_HTTP_OK,
+      .details.admin_add_incoming.credit_account_no = 2,
+      .details.admin_add_incoming.debit_account_no = 3,
+      .details.admin_add_incoming.exchange_base_url = "https://exchange.org/",
+      .details.admin_add_incoming.amount = "KUDOS:3.22" },
+    { .oc = TBI_OC_HISTORY,
+      .label = "history-2b",
+      .details.history.account_number = 2,
+      .details.history.direction = TALER_BANK_DIRECTION_BOTH,
+      .details.history.start_row_ref = NULL,
+      .details.history.num_results = 5 },
+    { .oc = TBI_OC_HISTORY,
+      .label = "history-2bi",
+      .details.history.account_number = 2,
+      .details.history.direction = TALER_BANK_DIRECTION_BOTH,
+      .details.history.start_row_ref = "debit-1",
+      .details.history.num_results = 5 },
     /* check transfers arrived at fakebank */
     { .oc = TBI_OC_EXPECT_TRANSFER,
-      .label = "expect-2",
-      .details.expect_transfer.cmd_ref = "deposit-2" },
+      .label = "expect-2d",
+      .details.expect_transfer.cmd_ref = "credit-2" },
+    { .oc = TBI_OC_EXPECT_TRANSFER,
+      .label = "expect-2c",
+      .details.expect_transfer.cmd_ref = "debit-2" },
     { .oc = TBI_OC_EXPECT_TRANSFER,
       .label = "expect-1",
-      .details.expect_transfer.cmd_ref = "deposit-1" },
+      .details.expect_transfer.cmd_ref = "debit-1" },
     /* check transfer list is now empty */
     { .oc = TBI_OC_EXPECT_TRANSFERS_EMPTY,
       .label = "expect-empty" },
