@@ -487,13 +487,12 @@ static const struct TALER_EXCHANGE_DenomPublicKey *
 find_pk (const struct TALER_EXCHANGE_Keys *keys,
          const struct TALER_Amount *amount)
 {
-  unsigned int i;
   struct GNUNET_TIME_Absolute now;
   struct TALER_EXCHANGE_DenomPublicKey *pk;
   char *str;
 
   now = GNUNET_TIME_absolute_get ();
-  for (i=0;i<keys->num_denom_keys;i++)
+  for (unsigned int i=0;i<keys->num_denom_keys;i++)
   {
     pk = &keys->denom_keys[i];
     if ( (0 == TALER_amount_cmp (amount,
@@ -504,7 +503,7 @@ find_pk (const struct TALER_EXCHANGE_Keys *keys,
   }
   /* do 2nd pass to check if expiration times are to blame for failure */
   str = TALER_amount_to_string (amount);
-  for (i=0;i<keys->num_denom_keys;i++)
+  for (unsigned int i=0;i<keys->num_denom_keys;i++)
   {
     pk = &keys->denom_keys[i];
     if ( (0 == TALER_amount_cmp (amount,
@@ -883,8 +882,9 @@ spend_coin (struct Coin *coin,
   dr.purpose.size = htonl (sizeof (struct TALER_DepositRequestPS));
   dr.purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_DEPOSIT);
   dr.h_contract_terms = h_contract_terms;
-  TALER_JSON_hash (merchant_details,
-		   &dr.h_wire);
+  GNUNET_assert (GNUNET_OK ==
+                 TALER_JSON_hash (merchant_details,
+                                  &dr.h_wire));
 
   dr.timestamp = GNUNET_TIME_absolute_hton (timestamp);
   dr.refund_deadline = GNUNET_TIME_absolute_hton (refund_deadline);
@@ -1188,12 +1188,11 @@ benchmark_run (void *cls)
  * @return #GNUNET_OK if the array is correctly built, #GNUNET_SYSERR
  * otherwise
  */
-static unsigned int
+static int
 build_refresh ()
 {
   char *amount_str;
   struct TALER_Amount amount;
-  unsigned int i;
   const struct TALER_EXCHANGE_DenomPublicKey *picked_denom;
   const struct TALER_EXCHANGE_Keys *keys;
 
@@ -1201,7 +1200,7 @@ build_refresh ()
 		     refresh_pk_len,
 		     0);
   keys = TALER_EXCHANGE_get_keys (exchange);
-  for (i=0; NULL != refresh_denoms[i]; i++)
+  for (unsigned int i=0; NULL != refresh_denoms[i]; i++)
   {
     GNUNET_asprintf (&amount_str,
 		     "%s:%s",
@@ -1234,10 +1233,12 @@ build_refresh ()
  *
  * @param cls closure
  * @param _keys information about keys of the exchange
+ * @param vc compatibility information
  */
 static void
 cert_cb (void *cls,
-         const struct TALER_EXCHANGE_Keys *_keys)
+         const struct TALER_EXCHANGE_Keys *_keys,
+	 enum TALER_EXCHANGE_VersionCompatibility vc)
 {
   /* check that keys is OK */
   if (NULL == _keys)
@@ -1290,7 +1291,6 @@ cert_cb (void *cls,
 static void
 do_shutdown (void *cls)
 {
-  unsigned int i;
   struct GNUNET_TIME_Relative duration;
 
   if (warm >= WARM_THRESHOLD)
@@ -1302,7 +1302,7 @@ do_shutdown (void *cls)
     GNUNET_SCHEDULER_cancel (benchmark_task);
     benchmark_task = NULL;
   }
-  for (i=0; i<nreserves; i++)
+  for (unsigned int i=0; i<nreserves; i++)
   {
     if (NULL != reserves[i].aih)
     {
@@ -1313,7 +1313,7 @@ do_shutdown (void *cls)
       reserves[i].aih = NULL;
     }
   }
-  for (i=0; i<COINS_PER_RESERVE * nreserves; i++)
+  for (unsigned int i=0; i<COINS_PER_RESERVE * nreserves; i++)
   {
     struct Coin *coin = &coins[i];
 
@@ -1436,8 +1436,6 @@ run (void *cls)
   char *bank_details_filename;
   char *merchant_details_filename;
   struct GNUNET_CRYPTO_EddsaPrivateKey *priv;
-  unsigned int i;
-  unsigned int j;
 
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "gotten pool_size of %d\n",
@@ -1517,7 +1515,7 @@ run (void *cls)
   ncoins = COINS_PER_RESERVE * nreserves;
   coins = GNUNET_new_array (ncoins,
                             struct Coin);
-  for (i=0;i < nreserves;i++)
+  for (unsigned int i=0;i < nreserves;i++)
   {
     struct Reserve *r = &reserves[i];
 
@@ -1525,7 +1523,7 @@ run (void *cls)
     GNUNET_CONTAINER_DLL_insert (empty_reserve_head,
 				 empty_reserve_tail,
 				 r);
-    for (j=0; j < COINS_PER_RESERVE; j++)
+    for (unsigned int j=0; j < COINS_PER_RESERVE; j++)
     {
       struct Coin *coin;
       unsigned int coin_index;
@@ -1601,7 +1599,12 @@ main (int argc,
   GNUNET_assert (COIN_VALUE <= (1LL << REFRESH_SLOTS_NEEDED));
   ret = GNUNET_GETOPT_run ("taler-exchange-benchmark",
 			   options, argc, argv);
-  GNUNET_assert (GNUNET_SYSERR != ret);
+  if (GNUNET_SYSERR == ret)
+  {
+    fprintf (stderr,
+	     "Invalid command line arguments\n");
+    return 1;
+  }
   if (GNUNET_NO == ret)
     return 0;
   if ( (0 != num_iterations) &&
