@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014, 2015 GNUnet e.V.
+  Copyright (C) 2014-2017 GNUnet e.V.
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -29,6 +29,17 @@
 #include "taler_signatures.h"
 #include "exchange_api_handle.h"
 
+/**
+ * Which revision of the Taler protocol is implemented
+ * by this library?  Used to determine compatibility.
+ */
+#define TALER_PROTOCOL_CURRENT 1
+
+/**
+ * How many revisions back are we compatible to?
+ */
+#define TALER_PROTOCOL_AGE 1
+
 
 /**
  * Log error related to CURL operations.
@@ -274,7 +285,6 @@ parse_json_denomkey (struct TALER_EXCHANGE_DenomPublicKey *denom_key,
   struct TALER_DenominationKeyValidityPS denom_key_issue;
   struct GNUNET_CRYPTO_RsaPublicKey *pk;
   struct GNUNET_CRYPTO_EddsaSignature sig;
-
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_fixed_auto ("master_sig",
 				 &sig),
@@ -309,7 +319,9 @@ parse_json_denomkey (struct TALER_EXCHANGE_DenomPublicKey *denom_key,
     return GNUNET_SYSERR;
   }
 
-  memset (&denom_key_issue, 0, sizeof (denom_key_issue));
+  memset (&denom_key_issue,
+          0,
+          sizeof (denom_key_issue));
   GNUNET_CRYPTO_rsa_public_key_hash (pk,
                                      &denom_key_issue.denom_hash);
   denom_key_issue.purpose.purpose
@@ -425,7 +437,7 @@ parse_json_auditor (struct TALER_EXCHANGE_AuditorInformation *auditor,
         GNUNET_JSON_parse (key,
                            kspec,
                            NULL, NULL))
-      {
+    {
       GNUNET_break_op (0);
       continue;
     }
@@ -460,6 +472,7 @@ parse_json_auditor (struct TALER_EXCHANGE_AuditorInformation *auditor,
     TALER_amount_hton (&kv.fee_refund,
                        &dk->fee_refund);
     kv.denom_hash = dk->h_key;
+
     if (GNUNET_OK !=
         GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_AUDITOR_EXCHANGE_KEYS,
                                     &kv.purpose,
@@ -485,29 +498,75 @@ parse_json_auditor (struct TALER_EXCHANGE_AuditorInformation *auditor,
  *
  * @param[in] resp_obj JSON object to parse
  * @param[out] key_data where to store the results we decoded
+ * @param[out] where to store version compatibility data
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on error (malformed JSON)
  */
 static int
 decode_keys_json (const json_t *resp_obj,
-                  struct TALER_EXCHANGE_Keys *key_data)
+                  struct TALER_EXCHANGE_Keys *key_data,
+		  enum TALER_EXCHANGE_VersionCompatibility *vc)
 {
   struct GNUNET_TIME_Absolute list_issue_date;
+  struct GNUNET_TIME_Absolute last_denom_issue_date;
   struct TALER_ExchangeSignatureP sig;
   struct TALER_ExchangeKeySetPS ks;
   struct GNUNET_HashContext *hash_context;
   struct TALER_ExchangePublicKeyP pub;
+  unsigned int age;
+  unsigned int revision;
+  unsigned int current;
 
-  memset (key_data, 0, sizeof (struct TALER_EXCHANGE_Keys));
   if (JSON_OBJECT != json_typeof (resp_obj))
+  {
+    GNUNET_break_op (0);
     return GNUNET_SYSERR;
-
-  hash_context = GNUNET_CRYPTO_hash_context_start ();
-  /* parse the master public key and issue date of the response */
+  }
+  /* check the version */
   {
     const char *ver;
     struct GNUNET_JSON_Specification spec[] = {
       GNUNET_JSON_spec_string ("version",
                                &ver),
+      GNUNET_JSON_spec_end()
+    };
+
+    if (GNUNET_OK !=
+        GNUNET_JSON_parse (resp_obj,
+                           spec,
+                           NULL, NULL))
+    {
+      GNUNET_break_op (0);
+      return GNUNET_SYSERR;
+    }
+    if (3 != sscanf (ver,
+		     "%u:%u:%u",
+		     &current,
+		     &revision,
+		     &age))
+    {
+      GNUNET_break_op (0);
+      return GNUNET_SYSERR;
+    }
+    *vc = TALER_EXCHANGE_VC_MATCH;
+    if (TALER_PROTOCOL_CURRENT < current)
+    {
+      *vc |= TALER_EXCHANGE_VC_NEWER;
+      if (TALER_PROTOCOL_CURRENT < current - age)
+	*vc |= TALER_EXCHANGE_VC_INCOMPATIBLE;
+    }
+    if (TALER_PROTOCOL_CURRENT > current)
+    {
+      *vc |= TALER_EXCHANGE_VC_OLDER;
+      if (TALER_PROTOCOL_CURRENT - TALER_PROTOCOL_AGE > current)
+	*vc |= TALER_EXCHANGE_VC_INCOMPATIBLE;
+    }
+    key_data->version = GNUNET_strdup (ver);
+  }
+
+  /* parse the master public key and issue date of the response */
+  hash_context = GNUNET_CRYPTO_hash_context_start ();
+  {
+    struct GNUNET_JSON_Specification spec[] = {
       GNUNET_JSON_spec_fixed_auto ("master_public_key",
                                    &key_data->master_pub),
       GNUNET_JSON_spec_fixed_auto ("eddsa_sig",
@@ -523,7 +582,6 @@ decode_keys_json (const json_t *resp_obj,
             GNUNET_JSON_parse (resp_obj,
                                spec,
                                NULL, NULL));
-    key_data->version = GNUNET_strdup (ver);
   }
 
   /* parse the signing keys */
@@ -550,7 +608,9 @@ decode_keys_json (const json_t *resp_obj,
     }
   }
 
-  /* parse the denomination keys */
+  /* parse the denomination keys, merging with the
+     possibly EXISTING array as required (/keys cherry picking) */
+  last_denom_issue_date.abs_value_us = 0LLU;
   {
     json_t *denom_keys_array;
     json_t *denom_key_obj;
@@ -559,48 +619,94 @@ decode_keys_json (const json_t *resp_obj,
     EXITIF (NULL == (denom_keys_array =
                      json_object_get (resp_obj, "denoms")));
     EXITIF (JSON_ARRAY != json_typeof (denom_keys_array));
-    if (0 == (key_data->num_denom_keys = json_array_size (denom_keys_array)))
-    {
-      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  "Found no denomination keys at this exchange\n");
-      goto EXITIF_exit;
-    }
-    key_data->denom_keys = GNUNET_new_array (key_data->num_denom_keys,
-                                             struct TALER_EXCHANGE_DenomPublicKey);
+
     index = 0;
     json_array_foreach (denom_keys_array, index, denom_key_obj) {
+      struct TALER_EXCHANGE_DenomPublicKey dk;
+      bool found = false;
+
       EXITIF (GNUNET_SYSERR ==
-              parse_json_denomkey (&key_data->denom_keys[index],
+              parse_json_denomkey (&dk,
                                    denom_key_obj,
                                    &key_data->master_pub,
                                    hash_context));
-    }
+      for (unsigned int j=0;j<key_data->num_denom_keys;j++)
+      {
+	if (0 == memcmp (&dk,
+			 &key_data->denom_keys[j],
+			 sizeof (dk)))
+	{
+	  found = true;
+	  break;
+	}
+      }
+      if (found)
+      {
+	/* 0:0:0 did not support /keys cherry picking */
+	GNUNET_break_op (0 == current);
+	continue;
+      }
+      if (key_data->denom_keys_size == key_data->num_denom_keys)
+	GNUNET_array_grow (key_data->denom_keys,
+			   key_data->denom_keys_size,
+			   key_data->denom_keys_size * 2 + 2);
+      key_data->denom_keys[key_data->num_denom_keys++] = dk;
+
+      /* Update "last_denom_issue_date" */
+      last_denom_issue_date
+        = GNUNET_TIME_absolute_max (last_denom_issue_date,
+                                    dk.valid_from);
+    };
   }
+  key_data->last_denom_issue_date = last_denom_issue_date;
 
   /* parse the auditor information */
   {
     json_t *auditors_array;
     json_t *auditor_info;
-    unsigned int len;
     unsigned int index;
 
     EXITIF (NULL == (auditors_array =
                      json_object_get (resp_obj, "auditors")));
     EXITIF (JSON_ARRAY != json_typeof (auditors_array));
-    len = json_array_size (auditors_array);
-    if (0 != len)
-    {
-      key_data->auditors = GNUNET_new_array (len,
-                                             struct TALER_EXCHANGE_AuditorInformation);
-      index = 0;
-      json_array_foreach (auditors_array, index, auditor_info) {
-        EXITIF (GNUNET_SYSERR ==
-                parse_json_auditor (&key_data->auditors[index],
-                                    auditor_info,
-                                    key_data));
+
+    /* Merge with the existing auditor information we have (/keys cherry picking) */
+    index = 0;
+    json_array_foreach (auditors_array, index, auditor_info) {
+      struct TALER_EXCHANGE_AuditorInformation ai;
+      bool found = false;
+
+      EXITIF (GNUNET_SYSERR ==
+              parse_json_auditor (&ai,
+                                  auditor_info,
+                                  key_data));
+      for (unsigned int j=0;j<key_data->num_auditors;j++)
+      {
+        struct TALER_EXCHANGE_AuditorInformation *aix = &key_data->auditors[j];
+	if (0 == memcmp (&ai.auditor_pub,
+			 &aix->auditor_pub,
+			 sizeof (struct TALER_AuditorPublicKeyP)))
+	{
+	  found = true;
+          /* Merge denomination key signatures of downloaded /keys into existing
+             auditor information 'aix'. */
+          GNUNET_array_grow (aix->denom_keys,
+                             aix->num_denom_keys,
+                             aix->num_denom_keys + ai.num_denom_keys);
+          memcpy (&aix->denom_keys[aix->num_denom_keys - ai.num_denom_keys],
+                  ai.denom_keys,
+                  ai.num_denom_keys * sizeof (struct TALER_EXCHANGE_DenomPublicKey *));
+	  break;
+	}
       }
-      key_data->num_auditors = len;
-    }
+      if (found)
+        continue; /* we are done */
+      if (key_data->auditors_size == key_data->num_auditors)
+	GNUNET_array_grow (key_data->auditors,
+			   key_data->auditors_size,
+			   key_data->auditors_size * 2 + 2);
+      key_data->auditors[key_data->num_auditors++] = ai;
+    };
   }
 
   /* Validate signature... */
@@ -612,7 +718,7 @@ decode_keys_json (const json_t *resp_obj,
   hash_context = NULL;
   EXITIF (GNUNET_OK !=
           TALER_EXCHANGE_test_signing_key (key_data,
-                                       &pub));
+					   &pub));
   EXITIF (GNUNET_OK !=
           GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_EXCHANGE_KEY_SET,
                                       &ks.purpose,
@@ -641,7 +747,7 @@ free_key_data (struct TALER_EXCHANGE_Keys *key_data)
   for (unsigned int i=0;i<key_data->num_denom_keys;i++)
     GNUNET_CRYPTO_rsa_public_key_free (key_data->denom_keys[i].key.rsa_public_key);
   GNUNET_array_grow (key_data->denom_keys,
-                     key_data->num_denom_keys,
+                     key_data->denom_keys_size,
                      0);
   for (unsigned int i=0;i<key_data->num_auditors;i++)
   {
@@ -651,7 +757,7 @@ free_key_data (struct TALER_EXCHANGE_Keys *key_data)
     GNUNET_free (key_data->auditors[i].auditor_url);
   }
   GNUNET_array_grow (key_data->auditors,
-                     key_data->num_auditors,
+                     key_data->auditors_size,
                      0);
   GNUNET_free_non_null (key_data->version);
   key_data->version = NULL;
@@ -672,14 +778,17 @@ request_keys (struct TALER_EXCHANGE_Handle *exchange);
  * not trigger download.
  *
  * @param exchange exchange to check keys for
+ * @param force_download #GNUNET_YES to force download even if /keys is still valid
  * @return until when the response is current, 0 if we are re-downloading
  */
 struct GNUNET_TIME_Absolute
-TALER_EXCHANGE_check_keys_current (struct TALER_EXCHANGE_Handle *exchange)
+TALER_EXCHANGE_check_keys_current (struct TALER_EXCHANGE_Handle *exchange,
+                                   int force_download)
 {
   if (NULL != exchange->kr)
     return GNUNET_TIME_UNIT_ZERO_ABS;
-  if (0 < GNUNET_TIME_absolute_get_remaining (exchange->key_data_expiration).rel_value_us)
+  if ( (GNUNET_NO == force_download) &&
+       (0 < GNUNET_TIME_absolute_get_remaining (exchange->key_data_expiration).rel_value_us) )
     return exchange->key_data_expiration;
   request_keys (exchange);
   return GNUNET_TIME_UNIT_ZERO_ABS;
@@ -703,12 +812,17 @@ keys_completed_cb (void *cls,
   struct TALER_EXCHANGE_Handle *exchange = kr->exchange;
   struct TALER_EXCHANGE_Keys kd;
   struct TALER_EXCHANGE_Keys kd_old;
+  enum TALER_EXCHANGE_VersionCompatibility vc;
 
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Received keys from URL `%s' with status %ld.\n",
               kr->url,
               response_code);
   kd_old = exchange->key_data;
+  memset (&kd,
+          0,
+          sizeof (struct TALER_EXCHANGE_Keys));
+  vc = TALER_EXCHANGE_VC_PROTOCOL_ERROR;
   switch (response_code)
   {
   case 0:
@@ -719,14 +833,52 @@ keys_completed_cb (void *cls,
       response_code = 0;
       break;
     }
+    /* We keep the denomination keys and auditor signatures from the
+       previous iteration (/keys cherry picking) */
+    kd.num_denom_keys = kd_old.num_denom_keys;
+    GNUNET_array_grow (kd.denom_keys,
+                       kd.denom_keys_size,
+                       kd.num_denom_keys);
+    /* First make a shallow copy, we then need another pass for the RSA key... */
+    memcpy (kd.denom_keys,
+            kd_old.denom_keys,
+            kd_old.num_denom_keys * sizeof (struct TALER_EXCHANGE_DenomPublicKey));
+    for (unsigned int i=0;i<kd_old.num_denom_keys;i++)
+      kd.denom_keys[i].key.rsa_public_key
+        = GNUNET_CRYPTO_rsa_public_key_dup (kd_old.denom_keys[i].key.rsa_public_key);
+
+    kd.num_auditors = kd_old.num_auditors;
+    kd.auditors = GNUNET_new_array (kd.num_auditors,
+                                    struct TALER_EXCHANGE_AuditorInformation);
+    /* Now the necessary deep copy... */
+    for (unsigned int i=0;i<kd_old.num_auditors;i++)
+    {
+      const struct TALER_EXCHANGE_AuditorInformation *aold = &kd_old.auditors[i];
+      struct TALER_EXCHANGE_AuditorInformation *anew = &kd.auditors[i];
+
+      anew->auditor_pub = aold->auditor_pub;
+      anew->auditor_url = GNUNET_strdup (aold->auditor_url);
+      GNUNET_array_grow (anew->denom_keys,
+                         anew->num_denom_keys,
+                         aold->num_denom_keys);
+      for (unsigned int j=0;j<aold->num_denom_keys;j++)
+      {
+        /* offsets will map 1:1 */
+        unsigned int off = aold->denom_keys[j] - kd_old.denom_keys;
+
+        GNUNET_assert (off < kd_old.num_denom_keys);
+        anew->denom_keys[j] = &kd.denom_keys[off];
+      }
+    }
+
     if (GNUNET_OK !=
         decode_keys_json (resp_obj,
-                          &kd))
+                          &kd,
+			  &vc))
     {
       response_code = 0;
       break;
     }
-    exchange->key_data = kd;
     json_decref (exchange->key_data_raw);
     exchange->key_data_raw = json_deep_copy (resp_obj);
     break;
@@ -736,6 +888,7 @@ keys_completed_cb (void *cls,
                 (unsigned int) response_code);
     break;
   }
+  exchange->key_data = kd;
 
   if (MHD_HTTP_OK != response_code)
   {
@@ -744,7 +897,8 @@ keys_completed_cb (void *cls,
     exchange->state = MHS_FAILED;
     /* notify application that we failed */
     exchange->cert_cb (exchange->cert_cb_cls,
-                       NULL);
+                       NULL,
+		       vc);
     if (NULL != exchange->key_data_raw)
       {
         json_decref (exchange->key_data_raw);
@@ -760,7 +914,8 @@ keys_completed_cb (void *cls,
   exchange->state = MHS_CERT;
   /* notify application about the key information */
   exchange->cert_cb (exchange->cert_cb_cls,
-                     &exchange->key_data);
+                     &exchange->key_data,
+		     vc);
   free_key_data (&kd_old);
 }
 
@@ -846,50 +1001,32 @@ static int
 parse_date_string (const char *date,
                    struct GNUNET_TIME_Absolute *at)
 {
-  static const char *const days[] =
-    { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
-  static const char *const mons[] =
-    { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
-      "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"  };
   struct tm now;
   time_t t;
-  char day[4];
-  char mon[4];
-  unsigned int i;
-  unsigned int mday;
-  unsigned int year;
-  unsigned int h;
-  unsigned int m;
-  unsigned int s;
+  const char *end;
 
-  if (7 != sscanf (date,
-                   "%3s, %02u %3s %04u %02u:%02u:%02u GMT",
-                   day,
-                   &mday,
-                   mon,
-                   &year,
-                   &h,
-                   &m,
-                   &s))
-    return GNUNET_SYSERR;
-  memset (&now, 0, sizeof (now));
-  now.tm_year = year - 1900;
-  now.tm_mday = mday;
-  now.tm_hour = h;
-  now.tm_min = m;
-  now.tm_sec = s;
-  now.tm_wday = 7;
-  for (i=0;i<7;i++)
-    if (0 == strcasecmp (days[i], day))
-      now.tm_wday = i;
-  now.tm_mon = 12;
-  for (i=0;i<12;i++)
-    if (0 == strcasecmp (mons[i], mon))
-      now.tm_mon = i;
-  if ( (7 == now.tm_wday) ||
-       (12 == now.tm_mon) )
+  memset (&now,
+          0,
+          sizeof (now));
+  end = strptime (date,
+                  "%a, %d %b %Y %H:%M:%S %Z", /* RFC-1123 standard spec */
+                  &now);
+  if ( (NULL == end) ||
+       ( (*end != '\n') &&
+         (*end != '\r') ) )
+  {
+    GNUNET_break_op (0);
     return GNUNET_SYSERR;
+  }
   t = mktime (&now);
+  if (((time_t) -1) == t)
+  {
+    GNUNET_log_strerror (GNUNET_ERROR_TYPE_ERROR,
+                         "mktime");
+    return GNUNET_SYSERR;
+  }
+  if (t < 0)
+    t = 0; /* can happen due to timezone issues if date was 1.1.1970 */
   at->abs_value_us = 1000LL * 1000LL * t;
   return GNUNET_OK;
 }
@@ -932,6 +1069,7 @@ header_cb (char *buffer,
                 "Failed to parse %s-header `%s'\n",
                 MHD_HTTP_HEADER_EXPIRES,
                 val);
+    kr->expire = GNUNET_TIME_UNIT_ZERO_ABS;
   }
   GNUNET_free (val);
   return total;
@@ -987,7 +1125,22 @@ request_keys (struct TALER_EXCHANGE_Handle *exchange)
   GNUNET_assert (NULL == exchange->kr);
   kr = GNUNET_new (struct KeysRequest);
   kr->exchange = exchange;
-  kr->url = MAH_path_to_url (exchange, "/keys");
+  if (GNUNET_YES == MAH_handle_is_ready (exchange))
+  {
+    char *arg;
+
+    GNUNET_asprintf (&arg,
+		     "/keys?last_issue_date=%llu",
+		     (unsigned long long) exchange->key_data.last_denom_issue_date.abs_value_us / 1000000LLU);
+    kr->url = MAH_path_to_url (exchange,
+			       arg);
+    GNUNET_free (arg);
+  }
+  else
+  {
+    kr->url = MAH_path_to_url (exchange,
+			       "/keys");
+  }
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Requesting keys with URL `%s'.\n",
               kr->url);
@@ -1059,11 +1212,10 @@ TALER_EXCHANGE_test_signing_key (const struct TALER_EXCHANGE_Keys *keys,
                                  const struct TALER_ExchangePublicKeyP *pub)
 {
   struct GNUNET_TIME_Absolute now;
-  unsigned int i;
 
   /* we will check using a tolerance of 1h for the time */
   now = GNUNET_TIME_absolute_get ();
-  for (i=0;i<keys->num_sign_keys;i++)
+  for (unsigned int i=0;i<keys->num_sign_keys;i++)
     if ( (keys->sign_keys[i].valid_from.abs_value_us <= now.abs_value_us + 60 * 60 * 1000LL * 1000LL) &&
          (keys->sign_keys[i].valid_until.abs_value_us > now.abs_value_us - 60 * 60 * 1000LL * 1000LL) &&
          (0 == memcmp (pub,
@@ -1086,9 +1238,7 @@ const struct TALER_EXCHANGE_DenomPublicKey *
 TALER_EXCHANGE_get_denomination_key (const struct TALER_EXCHANGE_Keys *keys,
                                      const struct TALER_DenominationPublicKey *pk)
 {
-  unsigned int i;
-
-  for (i=0;i<keys->num_denom_keys;i++)
+  for (unsigned int i=0;i<keys->num_denom_keys;i++)
     if (0 == GNUNET_CRYPTO_rsa_public_key_cmp (pk->rsa_public_key,
                                                keys->denom_keys[i].key.rsa_public_key))
       return &keys->denom_keys[i];
@@ -1107,9 +1257,7 @@ const struct TALER_EXCHANGE_DenomPublicKey *
 TALER_EXCHANGE_get_denomination_key_by_hash (const struct TALER_EXCHANGE_Keys *keys,
                                              const struct GNUNET_HashCode *hc)
 {
-  unsigned int i;
-
-  for (i=0;i<keys->num_denom_keys;i++)
+  for (unsigned int i=0;i<keys->num_denom_keys;i++)
     if (0 == memcmp (hc,
                      &keys->denom_keys[i].h_key,
                      sizeof (struct GNUNET_HashCode)))
@@ -1127,7 +1275,8 @@ TALER_EXCHANGE_get_denomination_key_by_hash (const struct TALER_EXCHANGE_Keys *k
 const struct TALER_EXCHANGE_Keys *
 TALER_EXCHANGE_get_keys (struct TALER_EXCHANGE_Handle *exchange)
 {
-  (void) TALER_EXCHANGE_check_keys_current (exchange);
+  (void) TALER_EXCHANGE_check_keys_current (exchange,
+                                            GNUNET_NO);
   return &exchange->key_data;
 }
 
@@ -1142,10 +1291,10 @@ TALER_EXCHANGE_get_keys (struct TALER_EXCHANGE_Handle *exchange)
 json_t *
 TALER_EXCHANGE_get_keys_raw (struct TALER_EXCHANGE_Handle *exchange)
 {
-  (void) TALER_EXCHANGE_check_keys_current (exchange);
+  (void) TALER_EXCHANGE_check_keys_current (exchange,
+                                            GNUNET_NO);
   return json_deep_copy (exchange->key_data_raw);
 }
 
 
-
 /* end of exchange_api_handle.c */
