@@ -690,7 +690,7 @@ deserialize_melt_data (const char *buf,
  *         if the inputs are invalid (i.e. denomination key not with this exchange).
  *         Otherwise, pointer to a buffer of @a res_size to store persistently
  *         before proceeding to #TALER_EXCHANGE_refresh_melt().
- *         Non-null results should be freed using #GNUNET_free().
+ *         Non-null results should be freed using GNUNET_free().
  */
 char *
 TALER_EXCHANGE_refresh_prepare (const struct TALER_CoinSpendPrivateKeyP *melt_priv,
@@ -1337,6 +1337,11 @@ TALER_EXCHANGE_refresh_melt (struct TALER_EXCHANGE_Handle *exchange,
                         "melt_coin", melt_coin,
                         "coin_evs", coin_evs,
                         "transfer_pubs", transfer_pubs);
+  if (NULL == melt_obj)
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
 
   /* and now we can at last begin the actual request handling */
   rmh = GNUNET_new (struct TALER_EXCHANGE_RefreshMeltHandle);
@@ -1723,6 +1728,11 @@ TALER_EXCHANGE_refresh_reveal (struct TALER_EXCHANGE_Handle *exchange,
                           GNUNET_JSON_from_data_auto (&md->melt_session_hash),
                           "transfer_privs",
                           transfer_privs);
+  if (NULL == reveal_obj)
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
 
   /* finally, we can actually issue the request */
   rrh = GNUNET_new (struct TALER_EXCHANGE_RefreshRevealHandle);
@@ -1753,10 +1763,10 @@ TALER_EXCHANGE_refresh_reveal (struct TALER_EXCHANGE_Handle *exchange,
                                    strlen (rrh->json_enc)));
   ctx = MAH_handle_to_context (rrh->exchange);
   rrh->job = GNUNET_CURL_job_add (ctx,
-                          eh,
-                          GNUNET_YES,
-                          &handle_refresh_reveal_finished,
-                          rrh);
+                                  eh,
+                                  GNUNET_YES,
+                                  &handle_refresh_reveal_finished,
+                                  rrh);
   return rrh;
 }
 
