@@ -104,10 +104,12 @@ parse_reserve_history (struct TALER_EXCHANGE_Handle *exchange,
   struct TALER_Amount total_out;
   size_t off;
 
-  TALER_amount_get_zero (currency,
-                         &total_in);
-  TALER_amount_get_zero (currency,
-                         &total_out);
+  GNUNET_assert (GNUNET_OK ==
+                 TALER_amount_get_zero (currency,
+                                        &total_in));
+  GNUNET_assert (GNUNET_OK ==
+                 TALER_amount_get_zero (currency,
+                                        &total_out));
   uuid_off = 0;
   for (off=0;off<history_length;off++)
   {
@@ -354,8 +356,13 @@ parse_reserve_history (struct TALER_EXCHANGE_Handle *exchange,
       }
       TALER_amount_hton (&rcc.closing_amount,
 			 &amount);
-      TALER_JSON_hash (rhistory[off].details.close_details.receiver_account_details,
-		       &rcc.h_wire);
+      if (GNUNET_OK !=
+          TALER_JSON_hash (rhistory[off].details.close_details.receiver_account_details,
+                           &rcc.h_wire))
+      {
+        GNUNET_break (0);
+        return GNUNET_SYSERR;
+      }
       rcc.wtid = rhistory[off].details.close_details.wtid;
       rcc.purpose.size = htonl (sizeof (rcc));
       rcc.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_RESERVE_CLOSED);
@@ -820,7 +827,18 @@ reserve_withdraw_payment_required (struct TALER_EXCHANGE_ReserveWithdrawHandle *
      total incoming and outgoing amounts */
   len = json_array_size (history);
   {
-    struct TALER_EXCHANGE_ReserveHistory rhistory[len];
+    struct TALER_EXCHANGE_ReserveHistory *rhistory;
+
+    /* Use heap allocation as "len" may be very big and thus this may
+       not fit on the stack. Use "GNUNET_malloc_large" as a malicious
+       exchange may theoretically try to crash us by giving a history
+       that does not fit into our memory. */
+    rhistory = GNUNET_malloc_large (sizeof (struct TALER_EXCHANGE_ReserveHistory) * len);
+    if (NULL == rhistory)
+    {
+      GNUNET_break (0);
+      return GNUNET_SYSERR;
+    }
 
     if (GNUNET_OK !=
         parse_reserve_history (wsh->exchange,
