@@ -31,12 +31,16 @@
 #include "taler-exchange-httpd_admin.h"
 #include "taler-exchange-httpd_deposit.h"
 #include "taler-exchange-httpd_refund.h"
-#include "taler-exchange-httpd_reserve.h"
+#include "taler-exchange-httpd_reserve_status.h"
+#include "taler-exchange-httpd_reserve_withdraw.h"
 #include "taler-exchange-httpd_payback.h"
-#include "taler-exchange-httpd_wire.h"
-#include "taler-exchange-httpd_refresh.h"
-#include "taler-exchange-httpd_tracking.h"
+#include "taler-exchange-httpd_refresh_link.h"
+#include "taler-exchange-httpd_refresh_melt.h"
+#include "taler-exchange-httpd_refresh_reveal.h"
+#include "taler-exchange-httpd_track_transfer.h"
+#include "taler-exchange-httpd_track_transaction.h"
 #include "taler-exchange-httpd_keystate.h"
+#include "taler-exchange-httpd_wire.h"
 #if HAVE_DEVELOPER
 #include "taler-exchange-httpd_test.h"
 #endif
@@ -312,12 +316,14 @@ handle_mhd_request (void *cls,
       &TEH_MHD_handler_static_response, MHD_HTTP_NOT_FOUND
     };
   struct TEH_RequestHandler *rh;
-  unsigned int i;
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Handling request for URL '%s'\n",
               url);
-  for (i=0;NULL != handlers[i].url;i++)
+  if (0 == strcasecmp (method,
+                       MHD_HTTP_METHOD_HEAD))
+    method = MHD_HTTP_METHOD_GET; /* treat HEAD as GET here, MHD will do the rest */
+  for (unsigned int i=0;NULL != handlers[i].url;i++)
   {
     rh = &handlers[i];
     if ( (0 == strcasecmp (url,
@@ -1009,37 +1015,55 @@ main (int argc,
   listen_fds = getenv ("LISTEN_FDS");
   if ( (NULL != listen_pid) &&
        (NULL != listen_fds) &&
-       (getpid() == strtol (listen_pid, NULL, 10)) &&
-       ( (1 == strtoul (listen_fds, NULL, 10)) ||
-         (2 == strtoul (listen_fds, NULL, 10)) ) )
+       (getpid() == strtol (listen_pid,
+                            NULL,
+                            10)) &&
+       ( (1 == strtoul (listen_fds,
+                        NULL,
+                        10)) ||
+         (2 == strtoul (listen_fds,
+                        NULL,
+                        10)) ) )
   {
     int flags;
 
     fh = 3;
-    flags = fcntl (fh, F_GETFD);
-    if ( (-1 == flags) && (EBADF == errno) )
+    flags = fcntl (fh,
+                   F_GETFD);
+    if ( (-1 == flags) &&
+         (EBADF == errno) )
     {
       fprintf (stderr,
                "Bad listen socket passed, ignored\n");
       fh = -1;
     }
     flags |= FD_CLOEXEC;
-    if (0 != fcntl (fh, F_SETFD, flags))
+    if ( (-1 != fh) &&
+         (0 != fcntl (fh,
+                      F_SETFD,
+                      flags)) )
       GNUNET_log_strerror (GNUNET_ERROR_TYPE_ERROR,
                            "fcntl");
 
-    if (2 == strtoul (listen_fds, NULL, 10))
+    if (2 == strtoul (listen_fds,
+                      NULL,
+                      10))
     {
       fh_admin = 4;
-      flags = fcntl (fh_admin, F_GETFD);
-      if ( (-1 == flags) && (EBADF == errno) )
+      flags = fcntl (fh_admin,
+                     F_GETFD);
+      if ( (-1 == flags) &&
+           (EBADF == errno) )
       {
         fprintf (stderr,
                  "Bad listen socket passed, ignored\n");
         fh_admin = -1;
       }
       flags |= FD_CLOEXEC;
-      if (0 != fcntl (fh_admin, F_SETFD, flags))
+      if ( (-1 != fh_admin) &&
+           (0 != fcntl (fh_admin,
+                        F_SETFD,
+                        flags)) )
         GNUNET_log_strerror (GNUNET_ERROR_TYPE_ERROR,
                              "fcntl");
     }
