@@ -118,7 +118,7 @@ struct Command
       /**
        * Subject of the transfer, set by the command.
        */
-      struct TALER_WireTransferIdentifierRawP wtid;
+      char *subject;
 
     } expect_transaction;
 
@@ -450,7 +450,7 @@ do_deposit (struct Command *cmd)
         plugin->insert_deposit (plugin->cls,
                                 session,
                                 &deposit)) ||
-       (GNUNET_OK !=
+       (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
         plugin->commit (plugin->cls,
                         session)) )
     ret = GNUNET_SYSERR;
@@ -565,7 +565,7 @@ interpreter (void *cls)
                                   cmd->details.expect_transaction.debit_account,
                                   cmd->details.expect_transaction.credit_account,
                                   cmd->details.expect_transaction.exchange_base_url,
-                                  &cmd->details.expect_transaction.wtid))
+                                  &cmd->details.expect_transaction.subject))
         {
           fail (cmd);
           return;
@@ -1125,6 +1125,12 @@ run (void *cls)
   struct TALER_DenominationPublicKey dpk;
 
   plugin = TALER_EXCHANGEDB_plugin_load (cfg);
+  if (NULL == plugin)
+  {
+    GNUNET_break (0);
+    result = 77;
+    return;
+  }
   if (GNUNET_OK !=
       plugin->create_tables (plugin->cls))
   {
@@ -1143,12 +1149,12 @@ run (void *cls)
   if ( (GNUNET_OK !=
         plugin->start (plugin->cls,
                        session)) ||
-       (GNUNET_OK !=
+       (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
         plugin->insert_denomination_info (plugin->cls,
                                           session,
                                           &dpk,
                                           &issue)) ||
-       (GNUNET_OK !=
+       (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
         plugin->commit (plugin->cls,
                         session)) )
     {
