@@ -1,6 +1,6 @@
 /*
    This file is part of TALER
-   Copyright (C) 2014, 2015 GNUnet e.V.
+   Copyright (C) 2014-2017 GNUnet e.V.
 
    TALER is free software; you can redistribute it and/or modify it under the
    terms of the GNU General Public License as published by the Free Software
@@ -1292,13 +1292,16 @@ interpret (struct PERF_TALER_EXCHANGEDB_interpreter_state *state)
         break;
 
       case PERF_TALER_EXCHANGEDB_CMD_START_TRANSACTION:
-        state->plugin->start (state->plugin->cls, state->session);
+        GNUNET_break (GNUNET_OK ==
+                      state->plugin->start (state->plugin->cls,
+                                            state->session));
         break;
 
       case PERF_TALER_EXCHANGEDB_CMD_COMMIT_TRANSACTION:
-        state->plugin->commit (state->plugin->cls, state->session);
+        GNUNET_break (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS ==
+                      state->plugin->commit (state->plugin->cls,
+                                             state->session));
         break;
-
       case PERF_TALER_EXCHANGEDB_CMD_ABORT_TRANSACTION:
         state->plugin->rollback (state->plugin->cls,
                                  state->session);
@@ -1331,15 +1334,15 @@ interpret (struct PERF_TALER_EXCHANGEDB_interpreter_state *state)
       case PERF_TALER_EXCHANGEDB_CMD_INSERT_DEPOSIT:
         {
           int deposit_index;
-          int ret;
+          enum GNUNET_DB_QueryStatus qs;
           struct TALER_EXCHANGEDB_Deposit *deposit;
 
           deposit_index = state->cmd[state->i].details.insert_deposit.index_deposit;
           deposit = state->cmd[deposit_index].exposed.data.deposit;
-          ret = state->plugin->insert_deposit (state->plugin->cls,
+          qs = state->plugin->insert_deposit (state->plugin->cls,
                                                         state->session,
                                                         deposit);
-          GNUNET_assert (GNUNET_SYSERR != ret);
+          GNUNET_assert (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs);
           state->cmd[state->i].exposed.data.deposit = deposit;
         }
         break;
@@ -1347,7 +1350,7 @@ interpret (struct PERF_TALER_EXCHANGEDB_interpreter_state *state)
       case PERF_TALER_EXCHANGEDB_CMD_GET_DEPOSIT:
         {
           unsigned int source_index;
-          int ret;
+          enum GNUNET_DB_QueryStatus ret;
           struct PERF_TALER_EXCHANGEDB_Data *data;
 
           source_index = state->cmd[state->i].details.get_deposit.index_deposit;
@@ -1355,7 +1358,7 @@ interpret (struct PERF_TALER_EXCHANGEDB_interpreter_state *state)
           ret = state->plugin->have_deposit (state->plugin->cls,
                                              state->session,
                                              data->data.deposit);
-          GNUNET_assert (GNUNET_SYSERR != ret);
+          GNUNET_assert (0 >= ret);
         }
         break;
 
@@ -1419,12 +1422,15 @@ interpret (struct PERF_TALER_EXCHANGEDB_interpreter_state *state)
           unsigned int reserve_index;
           struct TALER_EXCHANGEDB_ReserveHistory *history;
           struct PERF_TALER_EXCHANGEDB_Data *data;
+	  enum GNUNET_DB_QueryStatus qs;
 
           reserve_index = state->cmd[state->i].details.get_reserve_history.index_reserve;
           data = &state->cmd[reserve_index].exposed;
-          history = state->plugin->get_reserve_history (state->plugin->cls,
-                                                        state->session,
-                                                        &data->data.reserve->reserve.pub);
+          qs = state->plugin->get_reserve_history (state->plugin->cls,
+						   state->session,
+						   &data->data.reserve->reserve.pub,
+						   &history);
+	  GNUNET_assert (0 >= qs);
           GNUNET_assert (NULL != history);
           state->plugin->free_reserve_history (state->plugin->cls,
                                                history);
@@ -1443,7 +1449,7 @@ interpret (struct PERF_TALER_EXCHANGEDB_interpreter_state *state)
       case PERF_TALER_EXCHANGEDB_CMD_INSERT_DENOMINATION:
         {
           unsigned int denom_index;
-          int ret;
+          enum GNUNET_DB_QueryStatus ret;
           struct TALER_EXCHANGEDB_DenominationKeyIssueInformation *dki ;
 
           denom_index = state->cmd[state->i].details.insert_denomination.index_denom;
@@ -1452,23 +1458,23 @@ interpret (struct PERF_TALER_EXCHANGEDB_interpreter_state *state)
                                                          state->session,
                                                          &dki->denom_pub,
                                                          &dki->issue);
-          GNUNET_assert (GNUNET_SYSERR != ret);
+          GNUNET_assert (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == ret);
         }
         break;
 
       case PERF_TALER_EXCHANGEDB_CMD_GET_DENOMINATION:
         {
           unsigned int denom_index;
-          int ret;
+          enum GNUNET_DB_QueryStatus qs;
           struct PERF_TALER_EXCHANGEDB_Data *data;
 
           denom_index = state->cmd[state->i].details.get_denomination.index_denom;
           data = &state->cmd[denom_index].exposed;
-          ret = state->plugin->get_denomination_info (state->plugin->cls,
-                                                      state->session,
-                                                      &data->data.dki->denom_pub,
-                                                      &data->data.dki->issue);
-          GNUNET_assert (GNUNET_SYSERR != ret);
+          qs = state->plugin->get_denomination_info (state->plugin->cls,
+						     state->session,
+						     &data->data.dki->denom_pub,
+						     &data->data.dki->issue);
+          GNUNET_assert (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs);
         }
         break;
 
@@ -1490,31 +1496,31 @@ interpret (struct PERF_TALER_EXCHANGEDB_interpreter_state *state)
       case PERF_TALER_EXCHANGEDB_CMD_INSERT_WITHDRAW:
         {
           unsigned int coin_index;
-          int ret;
+          enum GNUNET_DB_QueryStatus qs;
           struct PERF_TALER_EXCHANGEDB_Coin *coin;
 
           coin_index = state->cmd[state->i].details.insert_withdraw.index_coin;
           coin = state->cmd[coin_index].exposed.data.coin;
-          ret = state->plugin->insert_withdraw_info (state->plugin->cls,
+          qs = state->plugin->insert_withdraw_info (state->plugin->cls,
                                                      state->session,
                                                      &coin->blind);
-          GNUNET_assert (GNUNET_SYSERR != ret);
+          GNUNET_assert (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs);
         }
         break;
 
       case PERF_TALER_EXCHANGEDB_CMD_GET_WITHDRAW:
         {
           unsigned int source_index;
-          int ret;
+          enum GNUNET_DB_QueryStatus qs;
           struct PERF_TALER_EXCHANGEDB_Data *data;
 
           source_index = state->cmd[state->i].details.get_denomination.index_denom;
           data = &state->cmd[source_index].exposed;
-          ret = state->plugin->get_withdraw_info (state->plugin->cls,
-                                                  state->session,
-                                                  &data->data.coin->blind.h_coin_envelope,
-                                                  &data->data.coin->blind);
-          GNUNET_assert (GNUNET_SYSERR != ret);
+          qs = state->plugin->get_withdraw_info (state->plugin->cls,
+						 state->session,
+						 &data->data.coin->blind.h_coin_envelope,
+						 &data->data.coin->blind);
+          GNUNET_assert (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs);
         }
         break;
 
@@ -1523,13 +1529,16 @@ interpret (struct PERF_TALER_EXCHANGEDB_interpreter_state *state)
           unsigned int coin_index;
           struct PERF_TALER_EXCHANGEDB_Coin *coin;
           struct TALER_EXCHANGEDB_TransactionList *transactions;
+	  enum GNUNET_DB_QueryStatus qs;
 
           coin_index = state->cmd[state->i].details.get_coin_transaction.index_coin;
           coin = state->cmd[coin_index].exposed.data.coin;
-          transactions = state->plugin->get_coin_transactions (state->plugin->cls,
-                                                               state->session,
-                                                               &coin->public_info.coin_pub);
-          GNUNET_assert (transactions != NULL);
+          qs = state->plugin->get_coin_transactions (state->plugin->cls,
+						     state->session,
+						     &coin->public_info.coin_pub,
+						     &transactions);
+	  GNUNET_assert (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs);
+	  GNUNET_assert (transactions != NULL);
           state->plugin->free_coin_transaction_list (state->plugin->cls,
                                                      transactions);
         }
@@ -1544,10 +1553,11 @@ interpret (struct PERF_TALER_EXCHANGEDB_interpreter_state *state)
           refresh_session = PERF_TALER_EXCHANGEDB_refresh_session_init ();
           GNUNET_CRYPTO_hash_create_random (GNUNET_CRYPTO_QUALITY_WEAK,
                                             hash);
-          state->plugin->create_refresh_session (state->session,
-                                                 state->session,
-                                                 hash,
-                                                 refresh_session);
+          GNUNET_assert (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT ==
+			 state->plugin->create_refresh_session (state->session,
+								state->session,
+								hash,
+								refresh_session));
           state->cmd[state->i].exposed.data.session_hash = hash;
           PERF_TALER_EXCHANGEDB_refresh_session_free (refresh_session);
           GNUNET_free (refresh_session);
@@ -1580,11 +1590,12 @@ interpret (struct PERF_TALER_EXCHANGEDB_interpreter_state *state)
           denom_index = state->cmd[state->i].details.insert_refresh_order.index_denom;
           session_hash = state->cmd[hash_index].exposed.data.session_hash;
           denom = state->cmd[denom_index].exposed.data.dki;
-          state->plugin->insert_refresh_order (state->plugin->cls,
-                                               state->session,
-                                               session_hash,
-                                               1,
-                                               &denom->denom_pub);
+          GNUNET_assert (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT ==
+			 state->plugin->insert_refresh_order (state->plugin->cls,
+							      state->session,
+							      session_hash,
+							      1,
+							      &denom->denom_pub));
 
         }
         break;
@@ -1607,18 +1618,18 @@ interpret (struct PERF_TALER_EXCHANGEDB_interpreter_state *state)
 
       case PERF_TALER_EXCHANGEDB_CMD_INSERT_REFRESH_COMMIT_COIN:
         {
-          int ret;
+          enum GNUNET_DB_QueryStatus qs;
           unsigned int hash_index;
           struct TALER_EXCHANGEDB_RefreshCommitCoin *refresh_commit;
 
           hash_index = state->cmd[state->i].details.insert_refresh_commit_coin.index_hash;
           refresh_commit = PERF_TALER_EXCHANGEDB_refresh_commit_coin_init ();
-          ret = state->plugin->insert_refresh_commit_coins (state->plugin->cls,
-                                                            state->session,
-                                                            state->cmd[hash_index].exposed.data.session_hash,
-                                                            1,
-                                                            refresh_commit);
-          GNUNET_assert (GNUNET_OK == ret);
+          qs = state->plugin->insert_refresh_commit_coins (state->plugin->cls,
+							   state->session,
+							   state->cmd[hash_index].exposed.data.session_hash,
+							   1,
+							   refresh_commit);
+          GNUNET_assert (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs);
         }
         break;
 
