@@ -361,9 +361,9 @@ struct TALER_CoinPublicInfo
 struct TALER_TrackTransferDetails
 {
   /**
-   * Hash of the contract.
+   * Hash of the proposal data.
    */
-  struct GNUNET_HashCode h_contract;
+  struct GNUNET_HashCode h_contract_terms;
 
   /**
    * Which coin was deposited?
@@ -380,11 +380,6 @@ struct TALER_TrackTransferDetails
    */
   struct TALER_Amount coin_fee;
 
-  /**
-   * Merchant's transaction identifier.
-   */
-  uint64_t transaction_id;
-
 };
 
 
@@ -395,7 +390,7 @@ struct TALER_TrackTransferDetails
  * @param coin_public_info the coin public info to check for validity
  * @return #GNUNET_YES if the coin is valid,
  *         #GNUNET_NO if it is invalid
- *         #GNUNET_SYSERROR if an internal error occured
+ *         #GNUNET_SYSERR if an internal error occured
  */
 int
 TALER_test_coin_valid (const struct TALER_CoinPublicInfo *coin_public_info);
@@ -414,7 +409,7 @@ struct TALER_TransferSecretP
   /**
    * Secret used to derive private inputs for refreshed coins.
    * Must be (currently) a hash as this is what
-   * #GNUNET_CRYPTO_ecc_ecdh() returns to us.
+   * GNUNET_CRYPTO_ecc_ecdh() returns to us.
    */
   struct GNUNET_HashCode key;
 };
@@ -500,7 +495,7 @@ TALER_link_derive_transfer_secret (const struct TALER_CoinSpendPrivateKeyP *coin
  *
  * @param trans_priv transfer private key
  * @param coin_pub coin public key
- * @param[out] secret set to the shared secret
+ * @param[out] transfer_secret set to the shared secret
  */
 void
 TALER_link_reveal_transfer_secret (const struct TALER_TransferPrivateKeyP *trans_priv,
@@ -515,7 +510,7 @@ TALER_link_reveal_transfer_secret (const struct TALER_TransferPrivateKeyP *trans
  *
  * @param trans_pub transfer private key
  * @param coin_priv coin public key
- * @param[out] secret set to the shared secret
+ * @param[out] transfer_secret set to the shared secret
  */
 void
 TALER_link_recover_transfer_secret (const struct TALER_TransferPublicKeyP *trans_pub,
