@@ -53,6 +53,11 @@ enum TALER_ErrorCode
    */
   TALER_EC_NOT_IMPLEMENTED = 3,
 
+  /**
+   * Exchange is badly configured and thus cannot operate.
+   */
+  TALER_EC_EXCHANGE_BAD_CONFIGURATION = 4,
+
   /* ********** generic error codes ************* */
 
   /**
@@ -137,6 +142,14 @@ enum TALER_ErrorCode
    */
   TALER_EC_COIN_HISTORY_DB_ERROR_INSUFFICIENT_FUNDS = 1010,
 
+  /**
+   * Internal logic error.  Some server-side function
+   * failed that really should not.
+   * This response is provided with HTTP status code
+   * MHD_HTTP_INTERNAL_SERVER_ERROR.
+   */
+  TALER_EC_INTERNAL_LOGIC_ERROR = 1011,
+
 
 
   /* ********** request-specific error codes ************* */
@@ -399,6 +412,11 @@ enum TALER_ErrorCode
    */
   TALER_EC_DEPOSIT_INVALID_WIRE_FORMAT_TYPE_UNSUPPORTED = 1217,
 
+  /**
+   * Timestamp included in deposit permission is intolerably far off
+   * with respect to the clock of the exchange.
+   */
+  TALER_EC_DEPOSIT_INVALID_TIMESTAMP = 1218,
 
   /**
    * The respective coin did not have sufficient residual value
@@ -440,14 +458,6 @@ enum TALER_ErrorCode
    */
   TALER_EC_REFRESH_MELT_DB_STORE_SESSION_ERROR = 1304,
 
-  /**
-   * The exchange failed to store refresh order data in the
-   * database.
-   * This response is provided with HTTP status code
-   * MHD_HTTP_INTERNAL_ERROR.
-   */
-  TALER_EC_REFRESH_MELT_DB_STORE_ORDER_ERROR = 1305,
-
   /**
    * The exchange failed to store commit data in the
    * database.
@@ -456,14 +466,6 @@ enum TALER_ErrorCode
    */
   TALER_EC_REFRESH_MELT_DB_STORE_COMMIT_ERROR = 1306,
 
-  /**
-   * The exchange failed to store transfer keys in the
-   * database.
-   * This response is provided with HTTP status code
-   * MHD_HTTP_INTERNAL_ERROR.
-   */
-  TALER_EC_REFRESH_MELT_DB_STORE_TRANSFER_ERROR = 1307,
-
   /**
    * The exchange is unaware of the denomination key that was
    * requested for one of the fresh coins.  This response is provided
@@ -903,6 +905,12 @@ enum TALER_ErrorCode
    */
   TALER_EC_PAYBACK_COIN_BALANCE_NEGATIVE = 1857,
 
+  /**
+   * The "have" parameter was not a natural number.
+   * This reponse is provied with an HTTP status code of
+   * MHD_HTTP_BAD_REQUEST.
+   */
+  TALER_EC_KEYS_HAVE_NOT_NUMERIC = 1900,
 
 
   /* *********** Merchant backend error codes ********* */
@@ -1248,7 +1256,8 @@ enum TALER_ErrorCode
   /**
    * The exchange gave conflicting information about a coin which has
    * been wire transferred.
-   * The response is provided with HTTP status code MHD_HTTP_INTERNAL_SERVER_ERROR.
+   * The response is provided with HTTP status code
+   * MHD_HTTP_INTERNAL_SERVER_ERROR (FIXME: bad choice!).
    */
   TALER_EC_TRACK_TRANSFER_CONFLICTING_REPORTS = 2408,
 
@@ -1257,6 +1266,14 @@ enum TALER_ErrorCode
    */
   TALER_EC_TRACK_TRANSFER_JSON_RESPONSE_ERROR = 2409,
 
+  /**
+   * The exchange charged a different wire fee than what it
+   * originally advertised, and it is higher.  The response
+   * is provied with an HTTP status of
+   * MHD_HTTP_INTERNAL_SERVER_ERROR (FIXME: bad choice!).
+   */
+  TALER_EC_TRACK_TRANSFER_JSON_BAD_WIRE_FEE = 2410,
+
   /**
    * The hash provided in the request of /map/in does not match
    * the contract sent alongside in the same request.
@@ -1282,6 +1299,44 @@ enum TALER_ErrorCode
    */
   TALER_EC_PROPOSAL_LOOKUP_NOT_FOUND = 2503,
 
+  /**
+   * The client specified an unknown instance for any of the /refund operations
+   */
+  TALER_EC_REFUND_INSTANCE_UNKNOWN = 2600,
+
+  /**
+   * The frontend gave an unknown order id to issue the refund to.
+   */
+  TALER_EC_REFUND_ORDER_ID_UNKNOWN = 2601,
+
+  /**
+   * The amount to be refunded is inconsistent: either is lower than
+   * the previous amount being awarded, or it is too big to be paid back.
+   * In this second case, the fault stays on the business dept. side.
+   */
+  TALER_EC_REFUND_INCONSISTENT_AMOUNT = 2602,
+
+  /**
+   * The backend encountered an error while trying to retrieve the
+   * payment data from database.  Likely to be an internal error.
+   */
+  TALER_EC_REFUND_LOOKUP_DB_ERROR = 2603,
+
+  /**
+   * The backend encountered an error while trying to retrieve the
+   * payment data from database.  Likely to be an internal error.
+   */
+  TALER_EC_REFUND_MERCHANT_DB_COMMIT_ERROR = 2604,
+
+  /**
+   * Payments are stored in a single db transaction; this error indicates
+   * that one db operation within that transaction failed.  This might involve
+   * storing of coins or other related db operations, like starting/committing
+   * the db transaction or marking a contract as paid.
+   */
+   TALER_EC_PAY_DB_STORE_PAYMENTS_ERROR = 2605,
+
+
   /* ********** /test API error codes ************* */
 
   /**
