@@ -206,7 +206,19 @@ struct TALER_EXCHANGE_Keys
   char *version;
 
   /**
-   * Length of the @e sign_keys array.
+   * Timestamp indicating the /keys generation.
+   */
+  struct GNUNET_TIME_Absolute list_issue_date;
+
+  /**
+   * Timestamp indicating the creation time of the last
+   * denomination key in /keys.
+   * Used to fetch /keys incrementally.
+   */
+  struct GNUNET_TIME_Absolute last_denom_issue_date;
+
+  /**
+   * Length of the @e sign_keys array (number of valid entries).
    */
   unsigned int num_sign_keys;
 
@@ -220,6 +232,68 @@ struct TALER_EXCHANGE_Keys
    */
   unsigned int num_auditors;
 
+  /**
+   * Actual length of the @e auditors array (size of allocation).
+   */
+  unsigned int auditors_size;
+
+  /**
+   * Actual length of the @e denom_keys array (size of allocation).
+   */
+  unsigned int denom_keys_size;
+
+};
+
+
+/**
+ * How compatible are the protocol version of the exchange and this
+ * client?  The bits (1,2,4) can be used to test if the exchange's
+ * version is incompatible, older or newer respectively.
+ */
+enum TALER_EXCHANGE_VersionCompatibility
+{
+
+  /**
+   * The exchange runs exactly the same protocol version.
+   */
+  TALER_EXCHANGE_VC_MATCH = 0,
+
+  /**
+   * The exchange is too old or too new to be compatible with this
+   * implementation (bit)
+   */
+  TALER_EXCHANGE_VC_INCOMPATIBLE = 1,
+
+  /**
+   * The exchange is older than this implementation (bit)
+   */
+  TALER_EXCHANGE_VC_OLDER = 2,
+
+  /**
+   * The exchange is too old to be compatible with
+   * this implementation.
+   */
+  TALER_EXCHANGE_VC_INCOMPATIBLE_OUTDATED
+  = TALER_EXCHANGE_VC_INCOMPATIBLE
+  | TALER_EXCHANGE_VC_OLDER,
+
+  /**
+   * The exchange is more recent than this implementation (bit).
+   */
+  TALER_EXCHANGE_VC_NEWER = 4,
+
+  /**
+   * The exchange is too recent for this implementation.
+   */
+  TALER_EXCHANGE_VC_INCOMPATIBLE_NEWER
+  = TALER_EXCHANGE_VC_INCOMPATIBLE
+  | TALER_EXCHANGE_VC_NEWER,
+
+  /**
+   * We could not even parse the version data.
+   */
+  TALER_EXCHANGE_VC_PROTOCOL_ERROR = 8
+
 };
 
 
@@ -230,10 +304,12 @@ struct TALER_EXCHANGE_Keys
  * @param cls closure
  * @param keys information about the various keys used
  *        by the exchange, NULL if /keys failed
+ * @param compat protocol compatibility information
  */
 typedef void
 (*TALER_EXCHANGE_CertificationCallback) (void *cls,
-                                         const struct TALER_EXCHANGE_Keys *keys);
+                                         const struct TALER_EXCHANGE_Keys *keys,
+					 enum TALER_EXCHANGE_VersionCompatibility compat);
 
 
 /**
@@ -291,10 +367,12 @@ TALER_EXCHANGE_get_keys (struct TALER_EXCHANGE_Handle *exchange);
  * not, trigger /keys download.
  *
  * @param exchange exchange to check keys for
+ * @param force_download #GNUNET_YES to force download even if /keys is still valid
  * @return until when the response is current, 0 if we are re-downloading
  */
 struct GNUNET_TIME_Absolute
-TALER_EXCHANGE_check_keys_current (struct TALER_EXCHANGE_Handle *exchange);
+TALER_EXCHANGE_check_keys_current (struct TALER_EXCHANGE_Handle *exchange,
+                                   int force_download);
 
 
 /**
