@@ -173,6 +173,11 @@
  */
 #define TALER_SIGNATURE_MERCHANT_PAYMENT_OK 1104
 
+/**
+ * Signature where the merchant confirms a refund increase
+ */
+#define TALER_SIGNATURE_MERCHANT_REFUND_OK 1105
+
 
 /*********************/
 /* Wallet signatures */
@@ -788,7 +793,7 @@ struct TALER_ExchangeKeyValidityPS
   struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
 
   /**
-   * Hash of the auditor's URL.
+   * Hash of the auditor's URL (including 0-terminator).
    */
   struct GNUNET_HashCode auditor_url_hash;
 
@@ -1270,6 +1275,25 @@ struct TALER_ReserveCloseConfirmationPS
 };
 
 
+/**
+ * Used by the merchant to confirm with a signature that the refund has
+ * been successfully done. Even though the frontend doesn't usually do crypto,
+ * this signature may turn useful in court.
+ */
+struct TALER_MerchantRefundConfirmationPS
+{
+  /**
+   * Set TALER_SIGNATURE_REFUND_OK.
+   */
+  struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
+
+  /**
+   * Hashed order id; in case frontend wants to check it.
+   */
+  struct GNUNET_HashCode h_order_id GNUNET_PACKED;
+
+};
+
 GNUNET_NETWORK_STRUCT_END
 
 #endif
