@@ -292,7 +292,10 @@ validate_iban (const char *iban)
 
   len = strlen (iban);
   if (len > 34)
+  {
+    GNUNET_break_op (0);
     return GNUNET_NO;
+  }
   strncpy (cc, iban, 2);
   strncpy (ibancpy, iban + 4, len - 4);
   strncpy (ibancpy + len - 4, iban, 4);
@@ -305,11 +308,14 @@ validate_iban (const char *iban)
                sizeof (country_table) / sizeof (struct table_entry),
                sizeof (struct table_entry),
                &cmp_country_code))
+  {
+    GNUNET_break_op (0);
     return GNUNET_NO;
+  }
   nbuf = GNUNET_malloc ((len * 2) + 1);
   for (i=0, j=0; i < len; i++)
   {
-    if (isalpha ((int) ibancpy[i]))
+    if (isalpha ((unsigned char) ibancpy[i]))
     {
       if (2 != snprintf(&nbuf[j],
                         3,
@@ -326,7 +332,7 @@ validate_iban (const char *iban)
     j++;
   }
   for (j=0;'\0' != nbuf[j];j++)
-    GNUNET_assert (isdigit(nbuf[j]));
+    GNUNET_assert (isdigit( (unsigned char) nbuf[j]));
   GNUNET_assert (sizeof(dividend) >= 8);
   remainder = 0;
   for (i=0; i<j; i+=16)
@@ -338,6 +344,7 @@ validate_iban (const char *iban)
                        &nread)))
     {
       GNUNET_free (nbuf);
+      GNUNET_break_op (0);
       return GNUNET_NO;
     }
     if (0 != remainder)
@@ -347,6 +354,7 @@ validate_iban (const char *iban)
   GNUNET_free (nbuf);
   if (1 == remainder)
     return GNUNET_YES;
+  GNUNET_break_op (0); /* checksum wrong */
   return GNUNET_NO;
 }
 
